 /***********************************************************
  *
  *   Program:    CalcSum
  *   Created:    24/12/02
  *   Author:     Vladimir Molotkov
	*   e-mail:     vmolot@inrne.bas.bg
	*
	*		Usage:      calcsum filename [-#] |	[name1]...[name#]
	*   Function:   Calculates sums of numbers occuring in a text file "filename";
	*               if option "#" is used, the program considers numbers
	*               occuring in a file as members of a table with # columns 
	*								and calculates the sum in each column, as well as the
  *								number of "rows" in the virtual table;
	*								optional arguments "name..." assign names
	*								for corresponding columns of the table,
	*								instead of default names "sum1",..."sum#"
  *
  ************************************************************
  *
  ************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <cmath>
#include <ctype.h>
#include <io.h>

//inline version of atof()
//MS realization of atol was used as template;
//for better effectivity the pointer will move through the parsed
//part of the string
inline  double a2d(const char *&nptr)
{
	char c;              /* current char */
	double total,frak;	 /* current total */
	char sign;	         /* if '-', then negative, otherwise positive */
	int exp;

	/* skip whitespace */
	while (isspace(*nptr) )
		++nptr;

	c = *nptr++;
	sign = c;	   /* save sign indication */
	if (c == '-' || c == '+')
		c = *nptr++;    /* skip sign */

	total = 0;

	while (isdigit(c)) {
    total = 10 * total + (c - '0');     /* accumulate digit */
    c = *nptr++;    /* get next char */
	}

	if (c == '.')
    c = *nptr++,frak=0.1;

	while (isdigit(c)) {
    total += frak*(c - '0'),frak/=10;     /* accumulate fractional digit */
    c = *nptr++;    /* get next char */
	}

	if (sign == '-')
    total = -total;

	if (c == 'e' || c == 'E')
    c = *nptr++;    /* skip "e" */
	else{
		nptr--;//Bug Fix (29 Apr 01)
		return total;
	}
	sign = c;	   /* save exp sign indication */
	if (c == '-' || c == '+')
    c = *nptr++;    /* skip sign */

	exp = 0;

	while (isdigit(c)) {
    exp = 10 * exp + (c - '0');     /* accumulate digit */
    c = *nptr++;    /* get next char */
	}

	nptr--;//Bug Fix (29 Apr 01)
	if (sign == '-')
    return total/pow(10,exp);//pow(double,int) is inline as well
	else
    return total*pow(10,exp);
}

//***********************************************
int main(int argc,char**argv){
	if(argc==1){
		printf(
			"\nUsage:  calcsum filename [-#]|[name1 ... name#]\n\n"
	    "Function:   Calculates sums of numbers occuring in a file \"filename\".\n"
	    "if option \"#\" is used, the program considers numbers\n"
	    "occuring in a file as members of a table with # columns\n"
	    "and calculates the sum in each column, as well as the\n"
  		"number of \"rows\" in the \"virtual table\";\n"
			"optional arguments \"name...\" assign names\n"
			"for corresponding columns of the table,\n"
			"instead of default names \"sum1\",...\"sum#\"\n"
		);
		return 0;
	}
	FILE*fi;
	char*buf,*ebuf,*pos;
	int N=1;//Number of "columns" to sum; 
	if(!(fi=fopen(argv[1],"rb"))){
		printf("File %s does not exists!\n",argv[1]);exit(1);
	}
	bool defaultnames=true;
	if(argc>2)
		if(sscanf(argv[2],"-%d",&N));
		else defaultnames=false,N=argc-2;
	if(N<1)N=1;
	int flength=filelength(_fileno(fi));
	if(flength==0)return 0;
	buf=new char[flength+1];
	fread((void*)buf,sizeof(char),flength,fi);
	if(ferror(fi)){
		perror("\nRead error");
		fclose(fi);
		exit(1);
	}
	fclose(fi);
	ebuf=buf+flength;
	*ebuf=0;
	pos=buf;
	double*sum=new double[N];
	int n=0,i;
	for(i=0;i<N;i++)sum[i]=0;
	i=0;
	while(pos<ebuf)
		if(isdigit(*pos)&&(pos==buf||isspace(*(pos-1))/*||(*(pos-1)!='=')&&ispunct(*(pos-1))*/))
			sum[(i++)%N]+=a2d(pos),n++;
		else pos++;
	if(N==1)
		defaultnames?
		printf("N=%d, sum=%2.3f\n",n,sum[0]):
		printf("N=%d, %s=%2.3f\n",n,argv[2],sum[0]);
	else{
		printf("N=%d,",n/N);
		for(i=0;i<N;i++)
			defaultnames?
			printf(" sum%d=%2.3f",i+1,sum[i]):
			printf(" %s=%2.3f",argv[2+i],sum[i]);
		printf("\n");
	}
	return 0;
}
