 /***********************************************************
  *
  *   Program:    Exetimer
	*   Version     2.0
  *   Created:    24/12/02
  *   Author:     Vladimir Molotkov
	*   e-mail:     vmolot@inrne.bas.bg
	*
	*		Usage:      exetimer [options] PROGRAM [cmd line options for PROGRAM]
	*								Works in WinNT/2000/XP only!
	*
	*   Function:   Calculates execution times for PROGRAM;
	*								By default, four times are calculated:
	*								clock -- wall-clock time for the process PROGRAM;
	*								total -- difference between endtime and starttime,
	*								calculated by Win32 API function GetProgramTimes();
	*								process -- pure process time (in fact, the sum of
	*               kerneltime and usertime calculated by GetProgramTimes();
	*								user -- usertime calculated by GetProgramTimes();
	*								exetime options "-c", "-t" and "-k" swich off
	*								clock (resp. total, resp. kernel) time from exetime's output,
	*								whereas "+s" adds one more time interval: process+, which
	*								equals Process time + startup time for the process PROGRAM
	*								At last, the option "-t#" sets timeout # secs. for PROGRAM
	*
  *
  ************************************************************
  *
  ************************************************************/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <process.h>
#include <time.h>
#include <string.h>

#define FORBID_SYSTEMTIME_CHANGE

#ifdef FORBID_SYSTEMTIME_CHANGE
#include "privilege.h"
#endif


int main(int argc, char **argv)
{
	if(argc==1){
		printf(
			"Usage: exetimer [options] PROGRAM [cmd line options for PROGRAM]\n\n"
			"Function:   Calculates execution times for PROGRAM;\n"
			"By default, four times are calculated:\n"
			"clock -- wall-clock time for the process PROGRAM;\n"
			"total -- difference between endtime and starttime,\n"
			"calculated by Win32 API function GetProgramTimes();\n"
			"process -- pure process time (in fact, the sum of\n"
			"kerneltime and usertime calculated by GetProgramTimes();\n"
			"user -- usertime calculated by GetProgramTimes();\n"
			"exetime options \"-c\", \"-t\" and \"-k\" swich off\n"
			"clock (resp. total, resp. kernel) time from exetime's output,\n"
			"whereas \"+s\" adds one more time interval: process+, which\n"
			"equals Process time + startup time for the process PROGRAM\n"
			"At last, the option \"-t#\" sets timeout # secs. for PROGRAM\n"
		);
		return 0;
	}
	clock_t start;
  double loadtime,totaltime,kerneltime,usertime,clocktime;
		//clocktime is calculated with clock(), whereas data for totaltime
		//kerneltime and usertime are taken from GetProcessTimes()
	bool total=true,total0=true,kernel=true,load=false;
	int i=0;
	double timeout=(double)INFINITE/1000;
	//Process exetime cmd options:
	while(++i<argc)
		if(sscanf(argv[i],"-c"))total0=true;
		else
		if(sscanf(argv[i],"-t"))total=false;
		else
		if(sscanf(argv[i],"-t%f",&timeout));
		else
		if(sscanf(argv[i],"+s"))load=true;
		else
		if(sscanf(argv[i],"-k"))kernel=false;
		else break;

	if(timeout<0)timeout=-timeout;

	char cmd[256];//loaded program and its command line:
	strcpy(cmd,argv[i]);//must be loaded program name
	for(int j=i+1;j<argc;j++)//programs cmd line options (if any)
		strcat(cmd," "),
		strcat(cmd,argv[j]);
	printf("Starting %s. Wait...\n",cmd);
	_flushall();
	STARTUPINFO si;
	ZeroMemory(&si,sizeof(si));
	si.cb=sizeof(si);
	PROCESS_INFORMATION pi;
	ZeroMemory(&pi,sizeof(pi));
	FILETIME startnans,exitnans,kernelnans,usernans;//in 100ns time units
	start = clock();
	if(!CreateProcess(NULL,cmd,NULL,NULL,TRUE,//FALSE,
		NORMAL_PRIORITY_CLASS|CREATE_SUSPENDED,//Suspend it until privileges will be adjusted
		NULL,NULL,&si,&pi)){
		printf("Cannot start %s!\n",argv[i]);
		exit(1);
	}
#ifdef FORBID_SYSTEMTIME_CHANGE
	//Forbid system time changes:
	HANDLE th;
	if(!OpenProcessToken(pi.hProcess,TOKEN_ADJUST_PRIVILEGES|TOKEN_QUERY,&th))
		printf("Cannot open access token\n");
	if(!SetPrivilege(th,SE_SYSTEMTIME_NAME,false))//This is "OK", but created process CHANGES system time! 
		printf("Cannot forbid system time changes!\n");
	CloseHandle(th);
#endif
	ResumeThread(pi.hThread);
	loadtime=((double)clock()-(double)start)/CLOCKS_PER_SEC;;
	//WaitForSingleObject(pi.hProcess,INFINITE);
	if(WaitForSingleObject(pi.hProcess,(DWORD)(unsigned)(1000*timeout))==WAIT_TIMEOUT){
		printf("Timeout! Process %s aborted!\n",argv[i]);
		TerminateProcess(pi.hProcess,1);
		goto CLOSE;
	}
  clocktime=((double)clock()-(double)start)/CLOCKS_PER_SEC;//Non-adequately small negative values while resetting System Time one hour less! Why?
	GetProcessTimes(pi.hProcess,&startnans,&exitnans,&kernelnans,&usernans);
  totaltime=(
		(double)(unsigned __int64)exitnans.dwLowDateTime+
    (((unsigned __int64)exitnans.dwHighDateTime)<<32)
		-(//Wrong results sometimes if one takes sum of differencies instead!
		(double)(unsigned __int64)startnans.dwLowDateTime+
    (((unsigned __int64)startnans.dwHighDateTime)<<32)
		)
		)/10000000;

	usertime=(double)usernans.dwLowDateTime/10000000;
	kerneltime=usertime+(double)kernelnans.dwLowDateTime/10000000;
	printf("Done!");
	if(total0)
		printf(" clock: %2.3f sec;", clocktime);
	if(total)
		printf(" total: %2.3f sec;", totaltime);
	if(load)
		printf(" process+: %2.3f sec;", kerneltime+loadtime);
	if(kernel)
		printf(" process: %2.3f sec;", kerneltime);
	printf(" user: %2.3f sec.\n", usertime);

	// Close process and thread handles.
CLOSE:
	CloseHandle( pi.hProcess );
	CloseHandle( pi.hThread );

	return 0;
}
